

      subroutine genrot(rot)                              
C
C     Modified from G. Bricogne's program genrot in the mrc package.
C     Generates the 60 icosahedral rotations and their matrices
C     expressed in the orthnormal frame based on three dyads .
C
C     Author: Steve Fuller 10/29/89
C
C     Modified: Robert Sinkovits 6/27/06 - general cleanup and bringing
C     in line with Fortran 90 standard
C     Modified: Robert Sinkovits 12/8/06 - use save attribute so that
C     rotation matrices are only calculated the first time that genrot
C     is called.

      implicit real*8 (a-h,o-z)

      real, intent(out) :: rot(3,3,60)

      real*4, save  :: local_rot(3,3,60)
      logical, save :: first_time = .true.

      real*8 drot(3,3)
      real*8 p3(3,3)/9*0.0d0/,p5(3,3)/9*0.0d0/
      real*8 p3inv(3,3)/9*0.0d0/,p5inv(3,3)/9*0.0d0/,tmat6(3,3)
      real*8 tmat1(3,3),tmat2(3,3),tmat3(3,3),tmat4(3,3),tmat5(3,3)
      real*8 r2y(3,3,2)/18*0.0d0/,r2x(3,3,2)/18*0.0d0/
      real*8 s3(3,3,3)/27*0.0d0/,s5(3,3,5)/45*0.0d0/

      if(first_time) then
         first_time = .false.
         pi     = acos(-1.0d0)
         tpiov3 = pi/1.5
         tpiov5 = pi/2.5
         
         root5 = sqrt(5.0)
         
         xx3  = root5/6.0
         cos3 = sqrt(0.5+xx3)
         sin3 = sqrt(0.5-xx3)
         
         xx5  = 0.5/root5
         cos5 = sqrt(0.5+xx5)
         sin5 = sqrt(0.5-xx5)
         
C     Generate y-dyad matrices
         do k = 1,3
            r2y(k,k,1) = 1.0
         enddo
         r2y(1,1,2) = -1.0
         r2y(2,2,2) =  1.0
         r2y(3,3,2) = -1.0
         
C     Generate 3-fold matrices
         theta = -tpiov3
         do k = 1,3
            theta = theta + tpiov3
            cosine = cos(theta)
            sine = sin(theta)
            s3(1,1,k) = 1.0
            s3(2,2,k) = cosine
            s3(3,2,k) = sine
            s3(2,3,k) = -sine
            s3(3,3,k) = cosine
         enddo
         
C     Generate 5-fold matrices
         theta = -tpiov5
         do k = 1,5
            theta  = theta+tpiov5
            cosine = cos(theta)
            sine   = sin(theta)
            s5(1,1,k) = cosine
            s5(2,1,k) = sine
            s5(1,2,k) = -sine
            s5(2,2,k) = cosine
            s5(3,3,k) = 1.0
         enddo
         
C     Generate x-dyad matrices
         do k = 1,3
            r2x(k,k,1) = 1.0
         enddo
         r2x(1,1,2) = 1.0
         r2x(2,2,2) = -1.0
         r2x(3,3,2) = -1.0
         
C     Generate  xz-threefold transition matrices
         p3(1,1) = cos3
         p3(3,1) = sin3
         p3(2,2) = 1.0
         p3(1,3) = -sin3
         p3(3,3) = cos3
         p3inv(1,1) = cos3
         p3inv(3,1) = -sin3
         p3inv(2,2) = 1.0
         p3inv(1,3) = sin3
         p3inv(3,3) = cos3
         
C     Generate  xz-fivefold transition matrices
         p5(1,1) = cos5
         p5(3,1) = sin5
         p5(2,2) = 1.0
         p5(1,3) = -sin5
         p5(3,3) = cos5
         p5inv(1,1) = cos5
         p5inv(3,1) = -sin5
         p5inv(2,2) = 1.0
         p5inv(1,3) = sin5
         p5inv(3,3) = cos5
         
C     Generate icosahedral rotation matrices. Note use of Fortran 90
C     array syntax and matmul intrinsic function.
         n = 0
         do l2 = 1,2
            tmat1 = matmul(r2y(1:3,1:3,l2),p5)
            do k5 = 1,5
               tmat2 = matmul(tmat1,s5(1:3,1:3,k5))
               tmat3 = matmul(tmat2,p5inv)
               do j2 = 1,2
                  tmat4 = matmul(tmat3,r2x(1:3,1:3,j2))
                  tmat5 = matmul(tmat4,p3)
                  do i3 = 1,3
                     n = n+1
                     tmat6 = matmul(tmat5,s3(1:3,1:3,i3))
                     drot = matmul(tmat6,p3inv)
                     local_rot(1:3,1:3,n) = drot(1:3,1:3)
                  enddo
               enddo
            enddo
         enddo
      endif

      rot = local_rot

      return
      end subroutine genrot
