      subroutine bcast_parameters

C     Subroutine: bcast_parameters
C     Author: Hong Lin (linh@cs.purdue.edu)
C     Date:   Apr.20, 1998
C
C     This routine broadcasts variables declared in the include files
C     allprogs.inc and info.inc from the root process to all other
C     processes. To minimize the number of communication events, the
C     data are packed into arrays before being sent and then unpacked
C     after being received.
C     
C     Modification history:                                         
C     - Hong Lin, Version 1.0, Aug.18, 1998                     
C     - Hong Lin, Version 2.0, Oct. 8, 1999                     
C     - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             
C     - RSS, general cleanup 2/8/2007

      implicit none

#ifdef MPI
      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'

      integer i, j, alloc_stat, count_int, count_real
      integer, allocatable :: tmp_int(:)
      real, allocatable :: tmp_real(:)

      call mpi_bcast (nfiles, 1, mpi_integer, 0,
     +     mpi_comm_world, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, 'bcast_parameters',
     +     'error in mpi_bcast ')
      
      count_int  = nfiles*2 + 26
      count_real = nfiles*7 + 12
      
      allocate(tmp_int(count_int), tmp_real(count_real), 
     $     stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop(progname, 'bcast_parameters',
     +        'Error in memory allocation')
      endif

c     Pack data into arrays before broadcasting
      if (iam == 0) then
         do i = 1, nfiles
            tmp_int(2*(i-1) + 1) = nids(i) 
            tmp_int(2*(i-1) + 2) = nbox(i) 
         enddo
         j = nfiles*2
         tmp_int(j+1)  = num_symm
         tmp_int(j+2)  = min_x
         tmp_int(j+3)  = max_x
         tmp_int(j+4)  = min_y
         tmp_int(j+5)  = max_y
         tmp_int(j+6)  = min_z
         tmp_int(j+7)  = max_z
         tmp_int(j+8)  = max_views
         tmp_int(j+9)  = npix_row
         tmp_int(j+10) = npix_col
         tmp_int(j+11) = num_den_pix
         tmp_int(j+12) = pixhd_endian
         tmp_int(j+13) = pixhd_htyp
         tmp_int(j+14) = pixhd_mode
         tmp_int(j+15) = ctfmode
         tmp_int(j+16) = filter
         tmp_int(j+17) = outmode
         tmp_int(j+18) = symm_code
         tmp_int(j+19) = view_perread 
         tmp_int(j+20) = nfiles 
         tmp_int(j+21) = box_out
         tmp_int(j+22) = nangle
         tmp_int(j+23) = funcmode
         tmp_int(j+24) = ncenter
         tmp_int(j+25) = quick_search
         tmp_int(j+26) = apo_border

         do i = 1, nfiles 
            tmp_real(7*(i-1) + 1) = pixsizs(i) 
            tmp_real(7*(i-1) + 2) = volts(i)
            tmp_real(7*(i-1) + 3) = delf_majs(i)
            tmp_real(7*(i-1) + 4) = delf_mins(i)
            tmp_real(7*(i-1) + 5) = ang_majs(i)
            tmp_real(7*(i-1) + 6) = ampfacs(i)
            tmp_real(7*(i-1) + 7) = cs_coeffs(i)
         enddo
         j = nfiles*7
         tmp_real(j+1)  = res_max
         tmp_real(j+2)  = res_min
         tmp_real(j+3)  = mapmagfactor
         tmp_real(j+4)  = resolution
         tmp_real(j+5)  = zero_fill
         tmp_real(j+6)  = tempfac
         tmp_real(j+7)  = ctf_ff1
         tmp_real(j+8)  = ctf_ff2
         tmp_real(j+9)  = dangle
         tmp_real(j+10) = funcweight
         tmp_real(j+11) = dcenter
         tmp_real(j+12) = pixsiz_out
      endif

c     broadcast integer data
      call mpi_bcast (tmp_int, count_int, mpi_integer, 0,
     1     mpi_comm_world, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, 'bcast_parameters',
     +     'error in mpi_bcast ')
      
c     broadcast real data
      call mpi_bcast (tmp_real, count_real, mpi_real, 0,
     1     mpi_comm_world, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, 'bcast_parameters',
     +     'error in mpi_bcast ')

c     unpack received data
      if (iam > 0) then
         do i = 1, nfiles
            nids(i) = tmp_int(2*(i-1) + 1)
            nbox(i) = tmp_int(2*(i-1) + 2)
         enddo
         j = nfiles*2
         num_symm     = tmp_int(j+1)
         min_x        = tmp_int(j+2)
         max_x        = tmp_int(j+3)
         min_y        = tmp_int(j+4)
         max_y        = tmp_int(j+5)
         min_z        = tmp_int(j+6)
         max_z        = tmp_int(j+7)
         max_views    = tmp_int(j+8)
         npix_row     = tmp_int(j+9)
         npix_col     = tmp_int(j+10)
         num_den_pix  = tmp_int(j+11)
         pixhd_endian = tmp_int(j+12)
         pixhd_htyp   = tmp_int(j+13)
         pixhd_mode   = tmp_int(j+14)
         ctfmode      = tmp_int(j+15)
         filter       = tmp_int(j+16)
         outmode      = tmp_int(j+17)
         symm_code    = tmp_int(j+18)
         view_perread = tmp_int(j+19)
         nfiles       = tmp_int(j+20)
         box_out      = tmp_int(j+21)
         nangle       = tmp_int(j+22)
         funcmode     = tmp_int(j+23)
         ncenter      = tmp_int(j+24)
         quick_search = tmp_int(j+25)
         apo_border   = tmp_int(j+26)
         
         do i = 1, nfiles 
            pixsizs(i)   = tmp_real(7*(i-1) + 1)
            volts(i)     = tmp_real(7*(i-1) + 2)
            delf_majs(i) = tmp_real(7*(i-1) + 3)
            delf_mins(i) = tmp_real(7*(i-1) + 4)
            ang_majs(i)  = tmp_real(7*(i-1) + 5)
            ampfacs(i)   = tmp_real(7*(i-1) + 6)
            cs_coeffs(i) = tmp_real(7*(i-1) + 7)
         enddo
         j = nfiles*7
         res_max      = tmp_real(j+1)
         res_min      = tmp_real(j+2)
         mapmagfactor = tmp_real(j+3)
         resolution   = tmp_real(j+4)
         zero_fill    = tmp_real(j+5)
         tempfac      = tmp_real(j+6)
         ctf_ff1      = tmp_real(j+7)
         ctf_ff2      = tmp_real(j+8)
         dangle       = tmp_real(j+9)
         funcweight   = tmp_real(j+10)
         dcenter      = tmp_real(j+11)
         pixsiz_out   = tmp_real(j+12)
      endif

      deallocate(tmp_real, tmp_int)
#endif
      
      return
      end subroutine bcast_parameters
