      subroutine ctf_temp(ctf3, idim1, idim2, pixsize, volt, focus_maj,
     $     focus_min, ang_maj, amp_fac, cs, tempfac)
      
C     Calcualte the CTF correction, adjusted for inverse temperature factor

      implicit none
      
      integer, intent(in) :: idim1, idim2
      real, intent(in) :: pixsize, focus_maj, focus_min, ang_maj,
     $     amp_fac, cs, volt, tempfac
      
      real, intent(out) :: ctf3(0:idim1, -idim2:idim2)

      integer   ix, iy, idim
      real      xsq, d_star, psi, phi
      real      focus, pi, radian, wavelength, x, y, tmp
      real      cc, beta, de, t

C     Declare function return types
      real ctf_func, focus_astig
      
      pi     = 4.0*atan2(1.0,1.0)
      radian = pi/180.0

C     Initialize chromatic aberration (Cc), energy spread (dE), and
C     illumination aperature (beta) to zero until estimates become
C     available. These quantities are used in the spatial and temporal
C     coherence calculations.

      cc   = 0.0
      beta = 0.0
      de   = 0.0

      idim = idim1*2+1
      
C     Calculate relativistic electron wavelength in nm
      wavelength =1.2264264967/sqrt(volt + 0.9784663543e-6 * volt**2)

C     Convert angle of focus major (relative to x-axis) to radians
      phi = ang_maj * radian

      tmp = 10.0/(idim*pixsize) ! convert to nm units

      do ix = -idim2, idim2
         x = real(ix)
         xsq = x**2
         do iy = 0, idim1
            y = real(iy)
            d_star = sqrt(xsq + y**2) * tmp
            
            if (focus_maj == focus_min) then
               focus = focus_maj
            else
               if(ix == 0 .and. iy == 0) then
                  psi = 0.0
               else
                  psi = atan2(x,y)
               endif
               focus = focus_astig(focus_maj, focus_min, psi, phi)
            endif
            
            ctf3(iy,ix) = ctf_func(d_star, pi,wavelength, focus,
     $           amp_fac, cs, cc, beta, de, volt)
            
            t = exp ((tempfac/400.0) * (d_star**2)) ! adjusted for nm

            ctf3(iy,ix) = abs(ctf3(iy,ix) / t)
         enddo
      enddo

      return
      end subroutine ctf_temp
